package dev.micle.totemofreviving.network;

import dev.micle.totemofreviving.TotemOfReviving;
import net.minecraft.entity.player.ServerPlayerEntity;
import net.minecraft.item.ItemStack;
import net.minecraft.network.PacketBuffer;
import net.minecraft.server.management.PlayerList;
import net.minecraft.stats.Stats;
import net.minecraft.util.Hand;
import net.minecraftforge.fml.network.NetworkEvent;

import java.util.function.Supplier;

import static dev.micle.totemofreviving.item.StrawTotemItem.*;

public class ChangeTargetPacket {
	private final Hand hand;
	
	public ChangeTargetPacket(final Hand hand) {
		this.hand = hand;
	}
	
	public static void encode(final ChangeTargetPacket packet, final PacketBuffer buffer) {
		Network.writeVersionInfo(buffer);
		buffer.writeEnum(packet.hand);
	}
	
	public static ChangeTargetPacket decode(final PacketBuffer buffer) {
		Network.checkVersion(buffer);
		return new ChangeTargetPacket(buffer.readEnum(Hand.class));
	}
	
	public static void handle(final ChangeTargetPacket packet, final Supplier<NetworkEvent.Context> contextSupplier) {
		final NetworkEvent.Context context = contextSupplier.get();
		context.enqueueWork(() -> {
			ServerPlayerEntity sender = context.getSender();
			if (sender == null) { return; }
			ItemStack totem = sender.getItemInHand(packet.hand);
			
			PlayerList playerList = TotemOfReviving.PROXY.getServer().getPlayerList();
			
			int targetIndex = totem.getOrCreateTag().getInt(TAG_TARGET_INDEX) + 1;
			if (targetIndex > playerList.getPlayerCount()-1) {
				targetIndex = 0;
			}
			ServerPlayerEntity target = playerList.getPlayers().get(targetIndex);
			
			totem.getOrCreateTag().putInt(TAG_TARGET_INDEX, targetIndex);
			totem.getOrCreateTag().putString(TAG_TARGET_UUID, target.getStringUUID());
			totem.getOrCreateTag().putInt(TAG_TARGET_CHARGE, target.getStats().getValue(Stats.CUSTOM.get(Stats.DEATHS)));
		});
		context.setPacketHandled(true);
	}
}
